

**************************************************
***************** IMPORT DATA  *******************
**************************************************

*Set directory
local root "C:...\" /*Insert main directory*/
clear all
cd "`root'Stata\" /*Stata files go to this subfolder*/
set more off


*IMPORT US DATA
clear
insheet using "US_data.txt",case
format time %tq
tsset time

foreach v in pcedefl core_pcedefl cpi core_cpi ur nairu ogap rgdp income hpi tdebt mdebt pce pce_dur pce_ndur pce_serv rpce rpce_dur rpce_ndur rpce_serv{
	rename `v' US_`v'
}

*Create % changes for inflation
foreach v in US_pcedefl US_core_pcedefl US_cpi US_core_cpi{
	gen l`v' = log(`v')
	gen dl`v' = (l`v'-l1.l`v')*100
	gen `v'_yoy = (l`v'-l4.l`v')*100
}

*Seasonally adjusted annual real GDP growth rate and ugap
gen US_grgdp = (US_rgdp/l1.US_rgdp)^4*100-100
gen US_ugap = US_ur-US_nairu

gen state="US"
gen float statenr=.
order state statenr, after(time)
tsset time
saveold US_data, version(12) replace


*IMPORT SPF FORECASTS from 2012 onwards
clear
insheet using "US_spf.txt",case
format time %tq
saveold US_spf, version(12) replace


*IMPORT GREENBOOK FORECASTS
clear
insheet using "US_nairu_green.txt",case
format time %tq
rename US_og_green US_og0
saveold US_nairu_green, version(12) replace


*SELECT FOMC MEETING DATES FROM GREENBOOK
clear
insheet using "US_greenbook.txt",case

gen time = yq(year,quarter)
format time %tq
order time,before(difdays)

drop if year<1984
bysort time: egen min_days= min(difdays)
gen min_days_dum = 1 if min_days==difdays
keep if min_days_dum==1

*Drop duplicates
sort time
by time:  gen dup = cond(_N==1,0,_n)
sum if dup>1
drop if dup>1

*Merge datasets
merge 1:1 time using US_nairu_green
drop if _merge==2
drop _merge

merge 1:1 time using US_spf
drop _merge

*Select relevant variables to keep
keep time US_grgdp0 US_cpi1 US_cpi2 US_ur0 US_nairu_green US_og0 spf*
saveold US_greenbook, version(12) replace


*Merge with Greenbook and SPF forecasts
clear
use US_data
merge 1:1 time using US_greenbook
drop _merge

*Link Greenbook NAIRU to CBO's Nairu before 1989q1
forvalues i=1/120 {
	replace US_nairu_green = US_nairu_green[_n+1] - (US_nairu[_n+1]-US_nairu[_n]) if time <tq(1989q1)
}

*Compute ugap0 for t and average inflation forecast over t+1 and t+2 for Greenbook and SPF
gen US_ugap0 = US_ur0-US_nairu_green
gen US_cpi_favg = (US_cpi1 + US_cpi2)/2
gen spf_ugap = spf_ur-spf_nairu
gen spf_cpi_favg = (spf_cpi1 + spf_cpi2)/2

*Link Greenbook to SPF to extend Taylor rule over 2012-2017 
foreach v in ugap og grgdp {
	gen `v'_greenspf = US_`v'0 if time<tq(2012q1)
	replace `v'_greenspf = spf_`v' if time>=tq(2012q1)
}
gen cpi_favg_greenspf = US_cpi_favg if time<tq(2012q1)
replace cpi_favg_greenspf = spf_cpi_favg if time>=tq(2012q1)

*Wu_xia shadow rate
replace wu_xia = intr if time <tq(2009q1)

tsset time
saveold US_data, version(12) replace

**************************************************************************************************************************************************************************************



*IMPORT DATA FOR THE US STATES

clear
insheet using "State_data.txt",case
format time %tq
xtset statenr time

*Merge with national variables
capture merge m:1 time using US_data, assert(match master) keep(match master)
drop _merge

*Generate remaining variables
gen income_pc= (income)/pop if statenr!=. /*In the original paper I used number of consumers!*/
gen hpinc = hpi/(income_pc)
gen hpinc_sa = hpi_sa/(income_pc)
gen lhpinc_sa = log(hpinc_sa)
gen pce_dsv = pce_dur + pce_serv
sum pce_dsv pce_dur pce_serv

*Deflate variables
xtset statenr time
foreach v in hpi_sa income pce pce_dur pce_ndur pce_serv pce_dsv tdebt{
	by statenr: gen r`v' = (`v')/cpi*100
}

*Ratios
foreach v in income gdp pce pce_dur pce_ndur pce_serv pce_dsv{
	by statenr: gen r`v'_pc= (r`v')/pop*1000
}

foreach v in tdebt mdebt{
	gen `v'inc = `v'/income*100
}

*Taking logs
foreach v in rtdebt rhpi_sa rincome rincome_pc rgdp rgdp_pc rpce rpce_dur rpce_ndur rpce_serv rpce_dsv {
	gen l`v' =  log(`v')
	gen dl`v' = (l`v' - l1.l`v')*100
}		

gen dtdebtinc = d.tdebtinc if !missing(l1.tdebtinc)

foreach v in homeown_sa ur hpinc_sa intrcm ltv forecl{
	gen d`v' = d.`v' if !missing(l1.`v')
}


*Housing wealth
*House prices rebased to 2000Q1
foreach v in hpi_sa{
	gen `v'2000=`v'
	forval i=1/50 {
		capture quie sum `v'2000 if statenr==`i' & time==tq(2000q1)
		capture local temp=r(mean)
		capture qui replace `v'2000=`v'2000*100/`temp' if statenr==`i'
	}
}

sort statenr time
gen rhw_sa=((homeown_sa*HU)* hpi_sa2000*med_hp2000/100)/cpi*100

xtset statenr time
foreach v in rhw_sa{
	gen l`v' =  log(`v')
	gen dl`v' = (l`v' - l1.l`v')*100
	gen `v'_pc = `v'/pop
	gen l`v'_pc = log(`v'_pc)
}


*Create % changes for inflation
foreach v in cpi core_cpi{
	by statenr: gen l`v' = log(`v')
	by statenr: gen dl`v' = (l`v'-l1.l`v')*100
	by statenr: gen `v'_yoy = (l`v'-l4.l`v')*100
}

*For robustness checks: NAIRU for the states as the average in the steady state - the 90s and whole period
bysort statenr: egen nairu = mean(ur) if time>=tq(1990q1) & time<=tq(1999q4)
bysort statenr: egen nairu_ext = mean(ur) if time>=tq(1976q1) & time<=tq(2017q4)
by statenr (time), sort: replace nairu = nairu[_n-1] if nairu >= . 
forvalues i=1/56 {
	by statenr (time), sort: replace nairu = nairu[_n+1] if nairu>= . 
}

**************************************************************************************************************************************************************************************


*********************************************************
***************** FILTERING THE DATA  *******************
*********************************************************

*OUTPUT GAP, UGAP, DEBT GAP

**************************************************
*******New method by Hamilton (2018, RESTAT)******
**************************************************

*Regression of the t+h forecast on a constant and the four most recent values

foreach w in trend cycle{
foreach v in tdebtinc mdebtinc ur lrgdp{
	gen `w'_`v'_ham =.
}
}

xtset statenr time
foreach v in tdebtinc mdebtinc ur lrgdp{
forval i= 1/50{
	capture reg f8.`v' l(0/3).`v' if statenr==`i', robust 
	capture predict yfit, xb
	capture predict e, resid
	capture replace trend_`v'_ham = l8.yfit if statenr==`i' & time<=tq(2017q4)
	capture replace cycle_`v'_ham = l8.e if statenr==`i' & time<=tq(2017q4)
	capture drop yfit e
}
}

rename cycle_tdebtinc_ham tdebtinc_gap
rename cycle_mdebtinc_ham mdebtinc_gap
rename cycle_ur_ham ugap_ham
rename cycle_lrgdp_ham ogap_ham



*Other ugaps
gen ugap = ur-nairu
gen ugap_ext = ur-nairu_ext
gen ugap_cbo = ur-US_nairu /*Using CBO's NAIRU*/

saveold States_data, version(12) replace


**************************************************************************************************************************************************************************************

